==> faiadashu-0.5.3.tar.gz: lib/questionnaires/model/item/src/response_model.dart <==
import 'package:fhir/r4.dart';

import '../../../../coding/coding.dart';
import '../../../questionnaires.dart';

/// Model a response item, which might consist of multiple answers.
class ResponseModel {
  /// The individual answers to this questionnaire item.
  List<QuestionnaireResponseAnswer?> answers = [];

  /// Reason why this response is empty.
  ///
  /// see [DataAbsentReason]
  Code? dataAbsentReason;
  final QuestionnaireItemModel itemModel;

  QuestionnaireResponseItem? get responseItem => itemModel.responseItem;
  set responseItem(QuestionnaireResponseItem? ri) =>
      itemModel.responseItem = ri;

  ResponseModel(this.itemModel) {
    final int? answerCount = responseItem?.answer?.length;
    if (answerCount != null && answerCount > 0) {
      answers = responseItem!.answer!;
    } else {
      answers = [null];
    }

    dataAbsentReason = responseItem?.extension_?.dataAbsentReason;
  }

  /// Is the response 'asked but declined'
  bool get isAskedButDeclined =>
      dataAbsentReason == dataAbsentReasonAskedButDeclinedCode;

  /// Update the response with all the answers which are not null.
  ///
  /// Sets the response in the related [QuestionnaireItemModel].
  void updateResponse() {
    final filledAnswers = answers
        .where((answer) => answer != null)
        .map<QuestionnaireResponseAnswer>((answer) => answer!)
        .toList(growable: false);

    itemModel.responseItem = (filledAnswers.isEmpty && dataAbsentReason == null)
        ? null
        : QuestionnaireResponseItem(
            linkId: itemModel.linkId,
            text: itemModel.questionnaireItem.text,
            extension_: (dataAbsentReason != null)
                ? [
                    FhirExtension(
                        url: dataAbsentReasonExtensionUrl,
                        valueCode: dataAbsentReason)
                  ]
                : null,
            // FHIR cannot have empty arrays.
            answer: filledAnswers.isEmpty ? null : filledAnswers);
  }

  /// Is this response invalid?
  ///
  /// This is currently entirely based on the [dataAbsentReason].
  bool get isInvalid {
    return dataAbsentReason == dataAbsentReasonAsTextCode;
  }

  void _ensureAnswerModel() {
    // TODO: this assumes only a single answer.
    answerModel(0);
  }

  Iterable<QuestionnaireErrorFlag>? get isComplete {
    // Non-existent answer models can be incomplete, e.g. if minOccurs is not met.
    _ensureAnswerModel();

    final markers = <QuestionnaireErrorFlag>[];

    for (final am in _cachedAnswerModels.values) {
      final marker = am.isComplete;
      if (marker != null) {
        markers.add(marker);
      }
    }

    return (markers.isNotEmpty) ? markers : null;
  }

  bool get isUnanswered {
    _ensureAnswerModel();

    return _cachedAnswerModels.values.any((am) => !am.isUnanswered);
  }

  final Map<int, AnswerModel> _cachedAnswerModels = <int, AnswerModel>{};

  /// Returns an [AnswerModel] for the nth answer to an overall response.
  ///
  /// Only [answerIndex] == 0 is currently supported.
  AnswerModel answerModel(int answerIndex) {
    if (_cachedAnswerModels.containsKey(answerIndex)) {
      return _cachedAnswerModels[answerIndex]!;
    }

    final AnswerModel? answerModel;

    switch (itemModel.questionnaireItem.type!) {
      case QuestionnaireItemType.choice:
      case QuestionnaireItemType.open_choice:
        answerModel = CodingAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.quantity:
      case QuestionnaireItemType.decimal:
      case QuestionnaireItemType.integer:
        answerModel = NumericalAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.string:
      case QuestionnaireItemType.text:
      case QuestionnaireItemType.url:
        answerModel = StringAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.date:
      case QuestionnaireItemType.datetime:
      case QuestionnaireItemType.time:
        answerModel = DateTimeAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.boolean:
        answerModel = BooleanAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.display:
      case QuestionnaireItemType.group:
        answerModel = StaticAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.attachment:
      case QuestionnaireItemType.unknown:
      case QuestionnaireItemType.reference:
        // Throwing an exception here would lead to breakage of filler.
        answerModel = UnsupportedAnswerModel(this, answerIndex);
    }

    _cachedAnswerModels[answerIndex] = answerModel;

    return answerModel;
  }
}

==> faiadashu-0.5.4-dev.4.tar.gz: lib/questionnaires/model/item/src/response_model.dart <==
import 'package:fhir/r4.dart';

import '../../../../coding/coding.dart';
import '../../../questionnaires.dart';

/// Model a response item, which might consist of multiple answers.
class ResponseModel {
  /// The individual answers to this questionnaire item.
  List<QuestionnaireResponseAnswer?> answers = [];

  /// Reason why this response is empty.
  ///
  /// see [DataAbsentReason]
  Code? dataAbsentReason;
  final QuestionnaireItemModel itemModel;

  QuestionnaireResponseItem? get responseItem => itemModel.responseItem;
  set responseItem(QuestionnaireResponseItem? ri) =>
      itemModel.responseItem = ri;

  ResponseModel(this.itemModel) {
    final int? answerCount = responseItem?.answer?.length;
    if (answerCount != null && answerCount > 0) {
      answers = responseItem!.answer!;
    } else {
      answers = [null];
    }

    dataAbsentReason = responseItem?.extension_?.dataAbsentReason;
  }

  /// Is the response 'asked but declined'
  bool get isAskedButDeclined =>
      dataAbsentReason == dataAbsentReasonAskedButDeclinedCode;

  /// Update the response with all the answers which are not null.
  ///
  /// Sets the response in the related [QuestionnaireItemModel].
  void updateResponse() {
    final filledAnswers = answers
        .where((answer) => answer != null)
        .map<QuestionnaireResponseAnswer>((answer) => answer!)
        .toList(growable: false);

    itemModel.responseItem = (filledAnswers.isEmpty && dataAbsentReason == null)
        ? null
        : QuestionnaireResponseItem(
            linkId: itemModel.linkId,
            text: itemModel.questionnaireItem.text,
            extension_: (dataAbsentReason != null)
                ? [
                    FhirExtension(
                      url: dataAbsentReasonExtensionUrl,
                      valueCode: dataAbsentReason,
                    )
                  ]
                : null,
            // FHIR cannot have empty arrays.
            answer: filledAnswers.isEmpty ? null : filledAnswers,
          );
  }

  /// Is this response invalid?
  ///
  /// This is currently entirely based on the [dataAbsentReason].
  bool get isInvalid {
    return dataAbsentReason == dataAbsentReasonAsTextCode;
  }

  void _ensureAnswerModel() {
    // TODO: this assumes only a single answer.
    answerModel(0);
  }

  Iterable<QuestionnaireErrorFlag>? get isComplete {
    // Non-existent answer models can be incomplete, e.g. if minOccurs is not met.
    _ensureAnswerModel();

    final markers = <QuestionnaireErrorFlag>[];

    for (final am in _cachedAnswerModels.values) {
      final marker = am.isComplete;
      if (marker != null) {
        markers.add(marker);
      }
    }

    return (markers.isNotEmpty) ? markers : null;
  }

  bool get isUnanswered {
    _ensureAnswerModel();

    return _cachedAnswerModels.values.any((am) => !am.isUnanswered);
  }

  final Map<int, AnswerModel> _cachedAnswerModels = <int, AnswerModel>{};

  /// Returns an [AnswerModel] for the nth answer to an overall response.
  ///
  /// Only [answerIndex] == 0 is currently supported.
  AnswerModel answerModel(int answerIndex) {
    if (_cachedAnswerModels.containsKey(answerIndex)) {
      return _cachedAnswerModels[answerIndex]!;
    }

    final AnswerModel? answerModel;

    switch (itemModel.questionnaireItem.type) {
      case QuestionnaireItemType.choice:
      case QuestionnaireItemType.open_choice:
        answerModel = CodingAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.quantity:
      case QuestionnaireItemType.decimal:
      case QuestionnaireItemType.integer:
        answerModel = NumericalAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.string:
      case QuestionnaireItemType.text:
      case QuestionnaireItemType.url:
        answerModel = StringAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.date:
      case QuestionnaireItemType.datetime:
      case QuestionnaireItemType.time:
        answerModel = DateTimeAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.boolean:
        answerModel = BooleanAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.display:
        answerModel = DisplayAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.group:
        answerModel = GroupAnswerModel(this, answerIndex);
        break;
      case QuestionnaireItemType.attachment:
      case QuestionnaireItemType.unknown:
      case QuestionnaireItemType.reference:
        // Throwing an exception here would lead to breakage of filler.
        answerModel = UnsupportedAnswerModel(this, answerIndex);
    }

    _cachedAnswerModels[answerIndex] = answerModel;

    return answerModel;
  }
}
